ATTRIBUTION = 'BAN'
LICENCE = 'ODbL 1.0'
LOG_QUERIES = True
LOG_NOT_FOUND = True
EXTRA_FIELDS = [
    {'key': 'citycode'},
]
FILTERS = ["type", "postcode", "citycode"]
HOUSENUMBERS_PAYLOAD_FIELDS = ['id']
